#!/usr/bin/perl
use strict; 
use warnings;  
use LWP::Simple;
use JSON;
use FindBin;
use lib $FindBin::Bin;
use RBAC;

my $delay = 2;
my $service_settings_data = ServiceUtil::getServiceSettingsData();

sub generateHealthCheckUrl {
    my $url = $_[0];
    my $index = index($url, "api");
    if($index > -1) {
        my $healthcheck_url = substr($url, 0, $index) . "api/healthcheck";
        print "$healthcheck_url\n";
        return $healthcheck_url;
    } else {
        die("Invalid url: " . $url, "\n");
    }
}

sub urlExists {
    my $url = $_[0];
    my $ua = LWP::UserAgent->new;
    $ua->agent("$0/0.1");

    # Pass request to the user agent and get a response back
    my $req = HTTP::Request->new(GET => $url);
    my $res = $ua->request($req);
    if ($res->is_success){
        return 1;
    }

    print "$url\n";
    print $res->as_string;

    return undef;
}

sub ensureServicesAreRunning {
	print "Ensure services are running\n";
    my $rbac_url = generateHealthCheckUrl($service_settings_data->{ServiceUrls}->{RbacServiceUrl});
    my $scheduler_url = generateHealthCheckUrl($service_settings_data->{ServiceUrls}->{SchedulerServiceUrl});
    my $configuration_url = generateHealthCheckUrl($service_settings_data->{ServiceUrls}->{ConfigurationServiceUrl});

    do
    {
        if (urlExists($rbac_url) && urlExists($scheduler_url) && urlExists($configuration_url))
        {
            return;
        }

        print "Retrying in $delay seconds\n";
        sleep ($delay);

    } while(1);
}

sub createStartupJobs {
	print "Creating startup jobs\n";
    my $token = RBAC::login();
    my $ua = LWP::UserAgent->new;
    $ua->agent("$0/0.1");

    my $url = $service_settings_data->{ServiceUrls}->{ConfigurationServiceUrl}."/startupjobs";

    # Create a request
    my $req = HTTP::Request->new(POST => $url);
    $req->header("token"=>$token);
    $req->header("Content-Length"=>0);

    # Pass request to the user agent and get a response back
    my $res = $ua->request($req);

    if ($res->is_success){
        print "success";       
    } else {
        my $content = JSON::decode_json($res->content);
        die($content->{result}->{_message}, "\n");
    }
}

ensureServicesAreRunning();    
createStartupJobs();
