#!/bin/bash
PROTECTION_HOME="/opt/netapp/protectionservice"
SQL_PASSWD=""
ADMINPASSWORD=`/usr/sbin/vmtoolsd --cmd 'info-get guestinfo.ovfEnv' |sed -n '/vplatform.auth.passwd.Vendor_Provider_Appliance/p' |awk '{gsub("oe:value="," ");gsub("/>"," ");gsub("\""," ");print $NF}'`
IS_CLOUD_DEPLOY=`/usr/sbin/vmtoolsd --cmd 'info-get guestinfo.ovfEnv' |sed -n '/vplatform.vcenter.register.cloudDeploy.Vendor_Provider_Appliance/p' |awk '{gsub("oe:value="," ");gsub("/>"," ");gsub("\""," ");print $NF}'`
passwd_unlock() {
    # Unlock password files
	chattr -i /etc/passwd
	lsattr  /etc/passwd
	chattr -ai /etc/shadow
	lsattr  /etc/shadow
}

installDebs() {
	DEBPATH=$1
	dpkg -iE $DEBPATH
	result=$?
	if [[ $result -ne 0 ]] ; then
		echo "Install $DEBPATH failed."
		return 1
	fi
	return 0
}

installRabbitMQ() {
	echo "RabbitMQ start   ${PROTECTION_HOME}"
	# Install RabbitMQ here

    debs=( erlang-base_23.2.1-1_amd64.deb erlang-base-hipe_23.2.1-1_amd64.deb erlang-asn1_23.2.1-1_amd64.deb erlang-mnesia_23.2.1-1_amd64.deb erlang-runtime-tools_23.2.1-1_amd64.deb erlang-crypto_23.2.1-1_amd64.deb erlang-public-key_23.2.1-1_amd64.deb erlang-ssl_23.2.1-1_amd64.deb	erlang-syntax-tools_23.2.1-1_amd64.deb erlang-diameter_23.2.1-1_amd64.deb erlang-xmerl_23.2.1-1_amd64.deb erlang-tftp_23.2.1-1_amd64.deb erlang-ftp_23.2.1-1_amd64.deb erlang-inets_23.2.1-1_amd64.deb erlang-edoc_23.2.1-1_amd64.deb erlang-eldap_23.2.1-1_amd64.deb erlang-erl-docgen_23.2.1-1_amd64.deb erlang-eunit_23.2.1-1_amd64.deb erlang-odbc_23.2.1-1_amd64.deb erlang-snmp_23.2.1-1_amd64.deb erlang-os-mon_23.2.1-1_amd64.deb erlang-parsetools_23.2.1-1_amd64.deb erlang-ssh_23.2.1-1_amd64.deb erlang-tools_23.2.1-1_amd64.deb erlang-nox_23.2.1-1_all.deb rabbitmq-server_3.8.11-1_all.deb )
	for customdeb in ${debs[@]}; do
		passwd_unlock
		installDebs ${PROTECTION_HOME}/packages/$customdeb
		if [[ $? -ne 0 ]] ; then
			return 1
		fi
	done


	if [ ! -f /etc/rabbitmq/rabbitmq-env.conf ]; then
		echo "coping rabbitmq conf file to /etc/rabbitmq"
		cp /opt/netapp/protectionservice/rabbitmq-env.conf /etc/rabbitmq
	fi

	rabbitmq-plugins enable rabbitmq_management

	service rabbitmq-server stop
	service rabbitmq-server restart
	sleep 5
	############# Add Admin User #############
	rabbitmqctl add_user admin ${ADMINPASSWORD}
	rabbitmqctl set_user_tags admin administrator
	############# Add JobManager User #############
	rabbitmqctl add_user jobmanager ${ADMINPASSWORD}
	echo "#############List Users################"
	rabbitmqctl list_users
	#echo "#############################"
	############# Add JobManager Virtual Host #############
	rabbitmqctl add_vhost jobschedulerhost
	############# Set permission for JobManager User on the virtual host #############
	rabbitmqctl set_permissions -p jobschedulerhost jobmanager ".*" ".*" ".*"
	############# Restart RabbitMq Service #############
	echo "#############List vhosts################"
	rabbitmqctl list_vhosts
	echo "#############################"
	service rabbitmq-server stop
	service rabbitmq-server start
	sleep 3
	echo "RabbitMQ end"
	return 0
}

installNginx() {
	echo "Installing nginx server"
	passwd_unlock
    dpkg -i ${PROTECTION_HOME}/nginx_1.20.1-1_buster_amd64.deb
	if [ $? = 1 ] ; then
		echo "failed to install nginx server. Exiting."
		return 1
	fi

	echo "nginx server installed."
	# Configure certificate in Nginx
	cp -rf /$HOME/certs /$HOME/dotnet/
	cp ${PROTECTION_HOME}/nginx.conf /etc/nginx/
	cp ${PROTECTION_HOME}/default /etc/nginx/conf.d/default.conf
	cp ${PROTECTION_HOME}/timeout.conf /etc/nginx/conf.d/timeout.conf
	echo "starting nginx server."
	/etc/init.d/nginx start
	if [ $? = 0 ] ; then
		echo "nginx server started."
		sleep 5
		nginx -t
		if [ $? = 0 ] ; then
			echo "Reloading nginx configuration."
			nginx -s reload
			echo "nginx configuration reloaded."
		else
			echo "Failed to reload nginx configuration"
		fi
	else
		pgrep -x nginx
		if [ $? = 1 ] ; then
			echo "Failed to start nginx. Trying to restart."
			/etc/init.d/nginx start
			if [ $? = 0 ] ; then
				echo "nginx server started."
				sleep 5
				pgrep -x nginx
				if [ $? = 1 ] ; then
					echo "Failed to restart nginx. Exiting."
					return 1
				else
					nginx -t
					if [ $? = 0 ] ; then
						echo "Reloading nginx configuration."
						nginx -s reload
						echo "nginx configuration reloaded."
					else
						echo "Failed to reload nginx configuration"
					fi
				fi
			fi
		fi
	fi
	return 0
}

installMySQL() {
	echo "MySQL start"

	# Install
	debconf-set-selections <<< "mysql-community-server mysql-server/default-auth-override select Use Legacy Authentication Method (Retain MySQL 5.x Compatibility)"
	export DEBIAN_FRONTEND=noninteractive
	debconf-set-selections <<< "mysql-community-server mysql-community-server/root-pass password $SQL_PASSWD"
	debconf-set-selections <<< "mysql-community-server mysql-community-server/re-root-pass password $SQL_PASSWD"
	passwd_unlock

	dpkg -i ${PROTECTION_HOME}/MySQLDependency/mysql-community-server_8.0.23-1debian10_amd64.deb
	if [ $? = 1 ] ; then
		echo "failed to install mysql server. Exiting."
		return 1
	fi

	# Configure
	echo "[mysqld]" >> /etc/mysql/my.cnf
	echo "max_connections=500" >> /etc/mysql/my.cnf
	echo "innodb_buffer_pool_size=1G" >> /etc/mysql/my.cnf
	echo "disable_log_bin" >> /etc/mysql/my.cnf

	# Start service
	service mysql start

	sleep 5

	# Create schema
	cd ${PROTECTION_HOME}
	for sql in `ls -1 *.sql`; do echo Executing SQL Script $sql; mysql -h localhost -u root -p$SQL_PASSWD < $sql; done;

	for sql in `ls -1 Upgrade/*.sql`; do echo Executing SQL Script $sql; mysql -h localhost -u root -p$SQL_PASSWD < $sql; done;

	echo "MySQL end"

	# Save password
	echo "Encrypting MySQL password"
	OUTPUT=$(java -jar /opt/netapp/controlplane/passwordInteracter/croproxy-passwordinteracter.jar --encrypt $SQL_PASSWD)
	if [ `echo $OUTPUT | grep "SUCCESS"` ] ; then
		ENCRYPTEDPASS=`echo $OUTPUT | sed 's/SUCCESS://g'`
		echo -n '{"user":"root","password":"'$ENCRYPTEDPASS'"}' > /etc/mysql/passwd.conf
		chmod 757 /etc/mysql/passwd.conf
		echo "Password written in file successfully"
	else
		echo "Failed to encrypt MySQL password"
	fi

	return 0
}
createSqlPassword() {
	SQL_PASSWD=$(cat /dev/urandom | tr -dc "a-zA-Z0-9!#$%^&*()_+?><" | fold -w 32 | head -n 1)
}

genCloudDeployFlagSQLFile() {
    echo "Cloud Deploy Flag : ${IS_CLOUD_DEPLOY}"

    if [ "$IS_CLOUD_DEPLOY" = "True" ] ; then
        DEPLOYMENT_MODE="ANF"
    else
        DEPLOYMENT_MODE="ONPREM"
    fi

    echo "DeploymentMode : ${DEPLOYMENT_MODE}"

    DeploymentModeSqlFileName="/opt/netapp/protectionservice/1.0.0.0019_DeploymentMode.sql"

    echo "INSERT INTO nsm.nsm_configuration (nsm_Configuration_Name, nsm_Configuration_Value)" >> $DeploymentModeSqlFileName
    echo "SELECT * FROM (SELECT 'DeploymentMode', '${DEPLOYMENT_MODE}') AS tmp" >> $DeploymentModeSqlFileName
    echo "WHERE NOT EXISTS (" >> $DeploymentModeSqlFileName
    echo "    SELECT 1 FROM nsm.nsm_configuration WHERE nsm_Configuration_Name = 'DeploymentMode'" >> $DeploymentModeSqlFileName
    echo ") LIMIT 1;" >> $DeploymentModeSqlFileName
}

do_start() {
	dpkg -s nginx | grep Status
	if [ $? = 1 ] ; then
		installNginx  >> /var/log/netapp/install/installPkg.log
		if [ $result_nginx = 1 ] ; then
			echo "Nginx installation fails. Returning" >> /var/log/netapp/install/installPkg.log
		fi
	fi

	dpkg -s mysql-community-server | grep Status
	if [ $? = 1 ] ; then
		createSqlPassword
        genCloudDeployFlagSQLFile
		installMySQL >> /var/log/netapp/install/installPkg.log
		if [ $result_mysql = 1 ] ; then
			echo "MYSQL installation fails. Returning" >> /var/log/netapp/install/installPkg.log
		fi
	fi

	dpkg -s rabbitmq-server | grep Status
	if [ $? = 1 ] ; then
		installRabbitMQ >> /var/log/netapp/install/installPkg.log
		if [ $result_rq = 1 ] ; then
			echo "Rabbitmq installation fails. Returning" >> /var/log/netapp/install/installPkg.log
		fi
	fi


}

main() {
		mkdir -p /var/log/netapp/install
		echo "Param found $PARAM"
		if [ "$PARAM" = "-check" ] ; then
			echo "skipping dependencies check"
		elif [ "$PARAM" = "-init" ] ; then
			do_start
		fi
}
PARAM=$1
main
