#!/usr/bin/perl
use strict; 
use warnings;  
use LWP::Simple;
use JSON;
use FindBin;
use lib $FindBin::Bin;
use RBAC;
use Time::Piece;

my $backupScheduledTime = $ARGV[0];
my $backupFolderPath = $ARGV[1];
my $backupRetentionCount = $ARGV[2];


sub ParseTime{
    my $scheduleTime;
    if ($_[0]){
        my $scheduleTime;
        my $error;
        eval{
            $scheduleTime = Time::Piece->strptime($_[0], '%l:%M %p');
            1;
        } or do {
                $error = $@ || 'Unknown failure';
                die("Invalid time format", "\n");

        };
        return $scheduleTime->hms; 
    }
    else{
        return undef;
    };
}

sub getPatchOperation {
    my $path = $_[0];
    my $value = $_[1];

    if(!$path || !$value){
        return undef;
    }
    else{
        return '{"op" : "replace", "path" : "'.$path.'", "value" : "'.$value.'"}';
    }
}

sub configureBackupJob {
    my $scheduledTime = ParseTime($_[0]); 
    my $folderPath = $_[1];
    my $retentionCount = $_[2];

    my $token = RBAC::login();
    my $ua = LWP::UserAgent->new;
    $ua->agent("$0/0.1");

    my $serviceSettingsData = ServiceUtil::getServiceSettingsData();
    my $url = $serviceSettingsData->{ServiceUrls}->{ConfigurationServiceUrl}."/startupjobs/databasebackupjob";

    my $content = '[ ';
    if($scheduledTime){
        $content = $content.getPatchOperation("BackupScheduleTime", $scheduledTime).' ,';
    }

    if($retentionCount){
        if ($retentionCount =~ /^\d+?$/) {
            $content = $content.getPatchOperation("RetentionCount", $retentionCount).' ,';
        }
        else{
            die ("Invalid retention count", "\n");
        }
    }

    if($folderPath){
        $content = $content.getPatchOperation("BackupFolderPath", $folderPath).'  ,';
    }

    # Remove trailling comma
    $content =~ s/,$//;
    $content = $content. " ]";
    # print $content;

    # Create a request
    my $req = HTTP::Request->new(PATCH => $url);
    $req->header("token"=>$token);
    $req->content_type('application/json');
    $req->content($content);

    # Pass request to the user agent and get a response back
    my $res = $ua->request($req);

    if ($res->is_success){
        print "success";
    }
    else{
        my $responseContent = JSON::decode_json($res->content);
        die($responseContent->{Result}->{_message}, "\n");
    }

}

configureBackupJob $backupScheduledTime, $backupFolderPath, $backupRetentionCount;